import { Injectable, Pipe } from '@angular/core';
import { Translate } from './translate';
/**
 * @private
 * The Translate pipe makes it easy to translate strings.
 *
 * @usage
 * Translate using the current language or language set through Translate.setLanguage
 * {{ 'Please enter your location' | translate }}
 *
 * Translate using a specific language
 * {{ 'Please enter your location' | translate:"de" }}
 */
export var TranslatePipe = (function () {
    function TranslatePipe(translate) {
        this.translate = {};
        this.translate = translate;
    }
    TranslatePipe.prototype.transform = function (value, args) {
        var lang;
        if (args.length > 0) {
            lang = args[0];
        }
        return this.translate.translate(value, lang);
    };
    TranslatePipe.prototype.supports = function (obj) { return true; };
    TranslatePipe.decorators = [
        { type: Pipe, args: [{ name: 'translate' },] },
        { type: Injectable },
    ];
    /** @nocollapse */
    TranslatePipe.ctorParameters = [
        { type: Translate, },
    ];
    return TranslatePipe;
}());
